/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes
#include "wolfssl/wolfcrypt/sha256.h"  
#include "wolfssl/wolfcrypt/logging.h" 
#include "wolfssl/wolfcrypt/ecc.h"  
#include "wolfssl/wolfcrypt/port/pic32/crypt_ecc_pukcl.h" 
#include "wolfssl/wolfcrypt/port/pic32/CryptoLib_Hardware_pb.h" 
#include "wolfssl/wolfcrypt/port/pic32/CryptoLib_Headers_pb.h" 
#include "wolfssl/wolfcrypt/port/pic32/crypt_pukcl_functions.h" 
#include "wolfssl/wolfcrypt/asn.h" 

//This Macro defines that ECC verification will happens using HW Crypto Engine OR Wolfssl Software Lib.
//#define MCHP_HW_CRYPTO_ECC_CALCULATION

int   tolower(int);
int   toupper(int);
typedef enum
{
    ATCA_SUCCESS                = 0x00, //!< Function succeeded.
    ATCA_CONFIG_ZONE_LOCKED     = 0x01,
    ATCA_DATA_ZONE_LOCKED       = 0x02,
    ATCA_INVALID_POINTER,
    ATCA_INVALID_LENGTH,
    ATCA_WAKE_FAILED            = 0xD0, //!< response status byte indicates CheckMac failure (status byte = 0x01)
    ATCA_CHECKMAC_VERIFY_FAILED = 0xD1, //!< response status byte indicates CheckMac failure (status byte = 0x01)
    ATCA_PARSE_ERROR            = 0xD2, //!< response status byte indicates parsing error (status byte = 0x03)
    ATCA_STATUS_CRC             = 0xD4, //!< response status byte indicates DEVICE did not receive data properly (status byte = 0xFF)
    ATCA_STATUS_UNKNOWN         = 0xD5, //!< response status byte is unknown
    ATCA_STATUS_ECC             = 0xD6, //!< response status byte is ECC fault (status byte = 0x05)
    ATCA_STATUS_SELFTEST_ERROR  = 0xD7, //!< response status byte is Self Test Error, chip in failure mode (status byte = 0x07)
    ATCA_FUNC_FAIL              = 0xE0, //!< Function could not execute due to incorrect condition / state.
    ATCA_GEN_FAIL               = 0xE1, //!< unspecified error
    ATCA_BAD_PARAM              = 0xE2, //!< bad argument (out of range, null pointer, etc.)
    ATCA_INVALID_ID             = 0xE3, //!< invalid device id, id not set
    ATCA_INVALID_SIZE           = 0xE4, //!< Count value is out of range or greater than buffer size.
    ATCA_RX_CRC_ERROR           = 0xE5, //!< CRC error in data received from device
    ATCA_RX_FAIL                = 0xE6, //!< Timed out while waiting for response. Number of bytes received is > 0.
    ATCA_RX_NO_RESPONSE         = 0xE7, //!< Not an error while the Command layer is polling for a command response.
    ATCA_RESYNC_WITH_WAKEUP     = 0xE8, //!< Re-synchronization succeeded, but only after generating a Wake-up
    ATCA_PARITY_ERROR           = 0xE9, //!< for protocols needing parity
    ATCA_TX_TIMEOUT             = 0xEA, //!< for Microchip PHY protocol, timeout on transmission waiting for master
    ATCA_RX_TIMEOUT             = 0xEB, //!< for Microchip PHY protocol, timeout on receipt waiting for master
    ATCA_TOO_MANY_COMM_RETRIES  = 0xEC, //!< Device did not respond too many times during a transmission. Could indicate no device present.
    ATCA_SMALL_BUFFER           = 0xED, //!< Supplied buffer is too small for data required
    ATCA_COMM_FAIL              = 0xF0, //!< Communication with device failed. Same as in hardware dependent modules.
    ATCA_TIMEOUT                = 0xF1, //!< Timed out while waiting for response. Number of bytes received is 0.
    ATCA_BAD_OPCODE             = 0xF2, //!< opcode is not supported by the device
    ATCA_WAKE_SUCCESS           = 0xF3, //!< received proper wake token
    ATCA_EXECUTION_ERROR        = 0xF4, //!< chip was in a state where it could not execute the command, response status byte indicates command execution error (status byte = 0x0F)
    ATCA_UNIMPLEMENTED          = 0xF5, //!< Function or some element of it hasn't been implemented yet
    ATCA_ASSERT_FAILURE         = 0xF6, //!< Code failed run-time consistency check
    ATCA_TX_FAIL                = 0xF7, //!< Failed to write
    ATCA_NOT_LOCKED             = 0xF8, //!< required zone was not locked
    ATCA_NO_DEVICES             = 0xF9, //!< For protocols that support device discovery (kit protocol), no devices were found
    ATCA_HEALTH_TEST_ERROR      = 0xFA, //!< random number generator health test error
    ATCA_ALLOC_FAILURE          = 0xFB, //!< Couldn't allocate required memory
    ATCA_USE_FLAGS_CONSUMED     = 0xFC, //!< Use flags on the device indicates its consumed fully
    ATCA_NOT_INITIALIZED        = 0xFD, //!< The library has not been initialized so the command could not be executed
} ATCA_STATUS;


void jump_to_application(void);
static void hex_to_lowercase(char *, size_t);
static void uint8_to_hex(uint8_t, char*);
static void hex_to_uppercase(char *, size_t);

ATCA_STATUS atcab_bin2hex(const uint8_t* bin, size_t bin_size, char* hex, size_t* hex_size);
ATCA_STATUS atcab_bin2hex_(const uint8_t* bin, size_t bin_size, char* hex, size_t* hex_size, bool is_pretty, bool is_space, bool is_upper);
ATCA_STATUS atcab_hex2bin(const char* ascii_hex, size_t ascii_hex_len, uint8_t* binary, size_t* bin_len);
ATCA_STATUS atcab_hex2bin_(const char* hex, size_t hex_size, uint8_t* bin, size_t* bin_size, bool is_space);
uint32_t  temp=0;
#define BOOT_START_ADDR                  (0x00000000)
#define APP_START_ADDR                   (0x0000B000)
#define SIGNATURE_START_ADDR             (0x0003FB00)
#define PUK_START_ADDR                   (0x00009F00)
#define APP_LEN_PRESENT_ADDR             (0x0003FD00)
#define APP_END_ADDR                     (0x0003FAFF)   

#define ATCA_SIG_SIZE               (64)  
#define PUCK_SIZE               (64)  
uint8_t PUCK[PUCK_SIZE];
#define ATCA_SHA_DIGEST_SIZE                (32)
uint8_t digest[ATCA_SHA_DIGEST_SIZE];
uint8_t signature[ATCA_SIG_SIZE];
#define ATCA_SHA_DIGEST_SIZE                (32)
#define SHA_DATA_MAX                        (64)
// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************

int main ( void )
{
    /* Initialize all modules */
    SYS_Initialize ( NULL );
    char displaystr[400];
    size_t displaylen;

    printf("\r\nEntered in Boot Loader application");
    
    while ( true )
    {
        /* Maintain state machines of all polled MPLAB Harmony modules. */
        
        //*********Start: Printing of Different Parameter from Firmware**********
        //Print the Application Signature
        memcpy(signature, (void* )SIGNATURE_START_ADDR, ATCA_SIG_SIZE);       
        displaylen = sizeof(displaystr);
        atcab_bin2hex(signature, ATCA_SIG_SIZE, displaystr, &displaylen);
        printf("\r\nApplication Signature from firmware:\r\n%s\r\n", displaystr);
        
        //Print the ECC Public key
        memcpy(PUCK, (void* )PUK_START_ADDR , PUCK_SIZE);       
        displaylen = sizeof(displaystr);
        atcab_bin2hex(PUCK, PUCK_SIZE, displaystr, &displaylen);
        printf("\r\nPublic key from firmware:\r\n%s\r\n", displaystr);
        
        //Print the Application length used for Hash        
        uint32_t *app_len_ptr = (uint32_t* )APP_LEN_PRESENT_ADDR;
        uint32_t app_len = *app_len_ptr;
        printf("\r\nApplication Length from firmware: %ld Bytes\r\n", app_len);
        if(!( (app_len > 0) && (app_len < (APP_END_ADDR-APP_START_ADDR)))) 
        {
            printf("\r\nApplication Length is not in Limits,Failed to Proceed");
        }
        //***************ENDS:Printing of Parameter from Firmware***************
        
        //**********START: Calculate the Hash of application********************
        int ret;
        wc_Sha256 sha256;
        if ((ret = wc_InitSha256(&sha256)) != 0) 
        {
             printf("\r\nSHA INIT FAILED");
             WOLFSSL_MSG("wc_InitSha256 failed");
        }
        else 
        {
            const byte *Addr = (void*)APP_START_ADDR;
            ret = wc_Sha256Update(&sha256, Addr, app_len);
            if(ret < 0)
            {
                printf("\r\nSHA 256 Update Failed");
            }
            else
            {
                ret = wc_Sha256GetHash(&sha256, digest);
                if(ret < 0)
                {
                    printf("\r\nSHA FINAL FAILED");
                }
                else
                {
                    ///Print the digest i.e. calculated hash of the Application
                    displaylen = sizeof(displaystr);
                    atcab_bin2hex(digest, ATCA_SHA_DIGEST_SIZE, displaystr, &displaylen);
                    printf("\r\nCalculated Application Digest(HASH):\r\n%s\r\n", displaystr);
                }
            }
        }
        //*************ENDS: HASH Calculation of Application ENDS***************
        
        //*************START:Verification of ECC signature of Application******* 
        if(!(ret < 0))
        {
            int verifyStatus = 0;
            byte RawKeyBuffer[65]; //import_x963 function required length of 65 bytes
            RawKeyBuffer[0] = 0x04; //This is required to mention compression; "compressed even" or "Odd" or "Uncompressed"  
            for(uint8_t i = 1 ; i < 65; i++)
            {
               RawKeyBuffer[i] = PUCK[i-1];
            }
            //Initialize the Key
            ecc_key pubKey;
            ret = wc_ecc_init(&pubKey);
            ret = wc_ecc_import_x963(RawKeyBuffer, 65, &pubKey);

            //Initialize the Signature
            mp_int r,s;        
            mp_read_unsigned_bin (&r, &signature[0], 32);
            mp_read_unsigned_bin (&s, &signature[32], 32);
            
#ifndef MCHP_HW_CRYPTO_ECC_CALCULATION 
            //Verify the signature
            ret = wc_ecc_verify_hash_ex(&r,&s, digest, ATCA_SHA_DIGEST_SIZE,&verifyStatus, &pubKey);
#else
            byte eccDsaSig[72];
            word32 eccDsaSigLen = 72;
            
            //encode with DSA
            ret = StoreECC_DSA_Sig(eccDsaSig,&eccDsaSigLen,&r,&s);
            
            //Wait for the Crypto RAM clear process
            while ((PUKCCSR & BIT_PUKCCSR_CLRRAM_BUSY) != 0);
            // Initialize PUKCC and perform self test
            crypt_pukcl_SelfTest();
            
            wc_CryptoInfo* info;
            int ctx = 0x01;
            info = (wc_CryptoInfo*)XMALLOC(sizeof(wc_CryptoInfo),heap, DYNAMIC_TYPE_ECC);        
            info->pk.eccverify.sig = (byte*)eccDsaSig;
            info->pk.eccverify.siglen = eccDsaSigLen;
            info->pk.eccverify.hash = digest;
            info->pk.eccverify.hashlen = ATCA_SHA_DIGEST_SIZE;
            info->pk.eccverify.key = &pubKey;
            info->pk.eccverify.res = &verifyStatus;
            
            //Verify the signature using HW Crypto
            ret = Crypt_ECC_HandleEccVerifyReq(0x01, info, (void*)&ctx);
#endif
            //***************** ENDS: ECC Signature verification********************

            //*****START: Jumping to Application Based on Signature Verification**** 
            if(ret < 0)
            {
                printf("\r\nECC Verification Function failed");

            }
            if(verifyStatus == 1)
            {
                printf("\r\nApplication Verification is successful");
                printf("\r\nwaiting for few sec. before jump to application");
                //LED2 will be Glowing for 5 sec. before jumping to Application 
                LED2_Clear(); //ON the LED2 
                for(uint32_t i = 0; i < 100000000; i++)
                {
                    //Here Delay is introduced before Jumping to Application
                    //wait for few sec...
                }
                LED2_Set(); //OFF the LED2
                jump_to_application();
            }
            else
            {
                printf("\r\nApplication Verification is Failed");
                printf("\r\nApplication can not be Run");
            }
        }
        //*****ENDS: Jumping to Application Based on Signature Verification*****
        while(1); //keep in the loop if anything fails
        SYS_Tasks ( );
    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}


void jump_to_application(void)
{
   
    uint32_t app_start_addr = *(uint32_t*)(APP_START_ADDR+4);

  //  printf("\r\nJumping to the application\r\n");
    /* Rebase the Stack Pointer */
    __set_MSP(*(uint32_t*)APP_START_ADDR);  
     /* Rebase the vector table base address */
    SCB->VTOR = ((uint32_t) APP_START_ADDR & SCB_VTOR_TBLOFF_Msk); 
    /* Jump to application Reset Handler in the application */
    asm("bx %0"::"r"(app_start_addr));
}
ATCA_STATUS atcab_bin2hex(const uint8_t* bin, size_t bin_size, char* hex, size_t* hex_size)
{
    return atcab_bin2hex_(bin, bin_size, hex, hex_size, true, true, true);
}

ATCA_STATUS atcab_bin2hex_(const uint8_t* bin, size_t bin_size, char* hex, size_t* hex_size, bool is_pretty, bool is_space, bool is_upper)
{
    size_t i;
    size_t cur_hex_size = 0;
    size_t max_hex_size;

    // Verify the inputs
    if (bin == NULL || hex == NULL || hex_size == NULL)
    {
        return ATCA_BAD_PARAM;
    }

    max_hex_size = *hex_size;
    *hex_size = 0;

    // Convert one byte at a time
    for (i = 0; i < bin_size; i++)
    {
        if (cur_hex_size > max_hex_size)
        {
            break;
        }
        if (i != 0)
        {
            if (is_pretty && (i % 16 == 0))
            {
                if (cur_hex_size + 2 > max_hex_size)
                {
                    return ATCA_SMALL_BUFFER;
                }
                memcpy(&hex[cur_hex_size], "\r\n", 2);
                cur_hex_size += 2;
            }
            else
            {
                if (is_space)
                {
                    if (cur_hex_size + 1 > max_hex_size)
                    {
                        return ATCA_SMALL_BUFFER;
                    }
                    hex[cur_hex_size] = ' ';
                    cur_hex_size += 1;
                }
            }
        }

        if (cur_hex_size + 2 > max_hex_size)
        {
            return ATCA_SMALL_BUFFER;
        }
        uint8_to_hex(bin[i], &hex[cur_hex_size]);
        cur_hex_size += 2;
    }

    if (is_upper)
    {
        hex_to_uppercase(hex, cur_hex_size);
    }
    else
    {
        hex_to_lowercase(hex, cur_hex_size);
    }

    *hex_size = cur_hex_size;
    if (cur_hex_size < max_hex_size)
    {
        // Since we have room add NULL as a convenience, but don't add to the
        // output size.
        hex[cur_hex_size] = 0;
    }

    return ATCA_SUCCESS;
}
static void uint8_to_hex(uint8_t num, char* hex_str)
{
    uint8_t nibble = (num >> 4) & 0x0F;

    if (nibble < 10)
    {
        *(hex_str++) = '0' + nibble;
    }
    else
    {
        *(hex_str++) = 'A' + (nibble - 10);
    }
    nibble = num & 0x0F;
    if (nibble < 10)
    {
        *(hex_str++) = '0' + nibble;
    }
    else
    {
        *(hex_str++) = 'A' + (nibble - 10);
    }
}

static void hex_to_lowercase(char *buffer, size_t length)
{
    size_t index;

    if ((buffer != NULL) && (length > 0))
    {
        for (index = 0; index < length; index++)
        {
            buffer[index] = tolower(buffer[index]);
        }
    }
}


static void hex_to_uppercase(char *buffer, size_t length)
{
    size_t index;

    if ((buffer != NULL) && (length > 0))
    {
        for (index = 0; index < length; index++)
        {
            buffer[index] = toupper(buffer[index]);
        }
    }
}

/*******************************************************************************
 End of File
*/

